﻿//https://discord.com/api/oauth2/authorize?client_id={application id}&permissions=0&scope=bot%20applications.commands
using System;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using System.Net;
using System.Threading.Tasks;
using Discord;
using Discord.WebSocket;
using Microsoft.Extensions.Configuration;
using Newtonsoft.Json;
using System.Text;

namespace silverworker_discord
{
    class Program
    {
        private DiscordSocketClient _client;

        IConfigurationRoot config = new ConfigurationBuilder()
            .AddJsonFile("appsettings.json", true, true)
            .Build();

        public static void Main(string[] args)
            => new Program().MainAsync().GetAwaiter().GetResult();
        private Task Log(LogMessage msg)
        {
            Console.WriteLine(msg.ToString());
            return Task.CompletedTask;
        }
        public async Task MainAsync()
        {
            _client = new DiscordSocketClient();

            _client.Log += Log;

            await _client.LoginAsync(TokenType.Bot, config["token"]);
            await _client.StartAsync();

            _client.Ready += () => Task.Run(() =>
            {
                Console.WriteLine("Bot is connected! going to sign up for message received and user joined in client ready");
                Shared.botChatterChannel = _client.GetChannel(ulong.Parse(config["botChatterChannel"])) as ISocketMessageChannel;
                Shared.announcementChannel = _client.GetChannel(ulong.Parse(config["announcementChannel"])) as ISocketMessageChannel;
                Shared.mtgChannel = _client.GetChannel(ulong.Parse(config["mtgChannel"])) as ISocketMessageChannel;

                _client.MessageReceived += MessageReceived;
                _client.UserJoined += UserJoined;
                _client.JoinedGuild += IJoined;
            });
            // Block this task until the program is closed.
            await Task.Delay(-1);
        }

        private async Task MessageReceived(SocketMessage messageParam)
        {
            var message = messageParam as SocketUserMessage;
            if (message == null) return;
            if (message.Author.Id == _client.CurrentUser.Id) return;

            Console.WriteLine($"{message.Channel}, {message.Content} (message id: {message.Id})");

            if (message.Author.IsWebhook)
            {
                if (message.Author.Username == "greasemonkey reward watcher")
                {
                    Console.WriteLine("heard greasemonkey, this is bananas");
                    var type = message.Content.Split("\n")[0].Substring("type: ".Length);
                    var subData = message.Content.Split("\n")[1].Substring("data: ".Length);
                    try
                    {
                        await twitchery.twitcherize(type, subData);
                    }
                    catch (Exception e)
                    {
                        await Shared.botChatterChannel.SendMessageAsync($"aaaadam!\n{JsonConvert.SerializeObject(e)}");
                    }
                }
            }
            else
            {
                if (message.Channel.Id == Shared.botChatterChannel.Id)
                {
                    if (message.Attachments?.Count > 0)
                    {
                        Console.WriteLine(message.Attachments.Count);
                        foreach (var att in message.Attachments)
                        {
                            Console.WriteLine(att.Url);
                            await WebRequest.Create("http://192.168.1.151:3001/shortcuts?display_url=" + att.Url).GetResponseAsync();
                        }
                    }
                }
                else
                {
                    //any channel, from a user
                    if (message.Content.StartsWith("!qrplz "))
                    {
                        Features.qrify(message.Content.Substring("!qrplz ".Length), message);
                    }
                    else
                    {
                        var wordLikes = message.Content.Split(' ', StringSplitOptions.TrimEntries);
                        var links = wordLikes?.Where(wl => Uri.IsWellFormedUriString(wl, UriKind.Absolute)).Select(wl => new Uri(wl));
                        if (links != null && links.Count() > 0)
                        {
                            foreach (var link in links)
                            {
                                if (link.Host == "vm.tiktok.com")
                                {
                                    Features.detiktokify(link, message);
                                }
                            }
                        }

                        if (message.Attachments?.Count > 0)
                        {
                            Console.WriteLine($"{message.Attachments.Count} attachments");
                            var appleReactions = false;
                            foreach (var att in message.Attachments)
                            {
                                if (att.Filename?.EndsWith(".heic") == true)
                                {
                                    Features.deheic(message, att);
                                    appleReactions = true;
                                }
                            }
                            if (appleReactions)
                            {
#pragma warning disable 4014 //the "you're not awaiting this" warning. yeah I know, that's the beauty of an async method lol
                                message.AddReactionAsync(new Emoji("\U0001F34F"));
#pragma warning restore 4014
                            }
                        }
                    }
                }
            }
        }
        private Task UserJoined(SocketGuildUser arg)
        {
            Console.WriteLine($"user joined: {arg.Nickname}. Guid: {arg.Guild.Id}. Channel: {arg.Guild.DefaultChannel}");
            var abbreviatedNickname = arg.Nickname;
            if (arg.Nickname.Length > 3)
            {
                abbreviatedNickname = arg.Nickname.Substring(0, arg.Nickname.Length / 3);
            }
            Console.WriteLine($"imma call him {abbreviatedNickname}");
            return arg.Guild.DefaultChannel.SendMessageAsync($"oh hey {abbreviatedNickname}- IPLAYTHESEALOFORICHALCOS <:ORICHALCOS:852749196633309194>");
        }
        

        private Task IJoined(SocketGuild arg)
        {
            arg.DefaultChannel.SendMessageAsync($"members of {arg.Name}, I'm starting with... THE SEAL OF ORICHALCOS!");
            return arg.DefaultChannel.SendMessageAsync($"<:ORICHALCOS:852749196633309194>");
        }
    }
}